#!/usr/bin/env python3
from sysrepo_api import sysrepo_api3 as api
import subprocess, os.path
from oslo_utils import netutils

cfg_file = '/etc/rsyslog.d/remote_syslog.conf'


class SysrepoSyslog(api.SysrepoAPI):

    def __init__(self,
                 app_name="syslog",
                 log_file=None,
                 log_level=None,
                 console_log=False,
                 indexes={"remote-servers": "id"},
                 namespace="/ucpe:config/syslog:syslog",
                 run_as_subscriber=False,
                 priority=0#,
                 #service_list=['sr-syslog']
                 ):
        api.SysrepoAPI.__init__(
            self,
            app_name=app_name,
            log_file=log_file,
            log_level=log_level,
            console_log=console_log,
            namespace=namespace,
            indexes=indexes,
            run_as_subscriber=run_as_subscriber,
            priority=priority#,
            #service_list=service_list
        )

    def _get_server_id(self, xpath):
        start = "[id='"
        end = "']"
        id = str(xpath)[xpath.find(start)+len(start):xpath.find(end)]
        return int(id)

    def _merge_change_set(self, change_set):
        self.logger.info("Merge configuration")

        r_cfg = self.get_running_config(full_path=False)

        if not change_set:
            self.logger.error(" There are no changes in change set")
            return r_cfg

        for i in range(0, len(change_set)):
            if not change_set[i]["xpath"].startswith(self.namespace):
                continue
            v_set = change_set[i]

            id = self._get_server_id(v_set["xpath"])
            if not id:
                self.logger.error("Invalid server-id")
                return None

            if v_set["operation"] is "MODIFIED" or v_set["operation"] is "CREATED":

                if v_set["xpath"].find("/enable") != -1:
                    if v_set['new_value'] and 'true' in str(v_set['new_value']).lower():
                        r_cfg["remote-servers"][id - 1]['enable'] = True
                    else:
                        r_cfg["remote-servers"][id - 1]['enable'] = False
                elif v_set["xpath"].find("/facility") != -1:
                    r_cfg["remote-servers"][id - 1]['facility'] = v_set["new_value"]
                elif v_set["xpath"].find("/ip-address") != -1:
                    r_cfg["remote-servers"][id - 1]['ip-address'] = v_set["new_value"]
                elif v_set["xpath"].find("/port") != -1:
                    r_cfg["remote-servers"][id - 1]['port'] = v_set["new_value"]
                elif v_set["xpath"].find("/severity") != -1:
                    r_cfg["remote-servers"][id - 1]['severity'] = v_set["new_value"]

        return r_cfg

    def apply_config(self, change_set=None, apply_startup=False):
        self.logger.info("Apply syslog configuration")

        if (apply_startup):
            r_cfg = self.get_startup_config(full_path=False)
        else:
            r_cfg = self._merge_change_set(change_set)

        if not r_cfg or not r_cfg["remote-servers"]:
            self.logger.error("Error getting syslog configuration from sysrepo")
            return False

        self.logger.debug("Apply syslog configuration: {}" .format(r_cfg))
        # backup current config
        backup_data = None
        if os.path.isfile(cfg_file):
            with open(cfg_file, "r") as file:
                backup_data = file.read()
                self.logger.debug("Current configuration is {}".format(backup_data))
                file.close()

        try:

            f_data = ''

            for rs in r_cfg["remote-servers"]:
                if rs["enable"] is True:
                    self.logger.info("Applying configuration for server {}".format(rs["id"]))

                    if "severity" not in rs or rs["severity"] == 'all':
                        prio = '*'
                    else:
                        prio = rs["severity"]

                    if "facility" not in rs or rs["facility"] == 'all':
                        fac = '*'
                    else:
                        fac = rs["facility"]

                    if netutils.is_valid_ipv6(rs["ip-address"]) is True:
                        #IPv6
                        r_server = '{}.{} @[{}]:{}\n'.format(fac, prio, rs["ip-address"], rs["port"])
                    else:
                        #IPv4
                        r_server = '{}.{} @{}:{}\n'.format(fac, prio, rs["ip-address"], rs["port"])
                        f_data = f_data + r_server

            with open(cfg_file, 'w+') as f:
                f.truncate()
                f.seek(0)
                f.write(f_data)

            if apply_startup is False:
                subprocess.check_call(['systemctl', '--no-block', 'restart', 'rsyslog.service'])

        except Exception as ex:
            self.logger.error("Failed to apply syslog configuration!\n{}".format(ex))
            if backup_data and os.path.isfile(cfg_file):
                f_backup = open(cfg_file, "w+")
                f_backup.truncate()
                f_backup.seek(0)
                f_backup.write(backup_data)
                f_backup.close()
                self.logger.info("Previous Syslog configuration is restored")

            return False

        return True

    def set_remote_server(self, id):
        self.set_item("{}/remote-servers[id='{}']".format(self.namespace, id), id)

    def get_remote_server(self, id):
        return self.get_item("{}/remote-servers[id='{}']".format(self.namespace, id))

    def unset_remote_server(self, id):
        self.delete_item("{}/remote-servers[id='{}']".format(self.namespace, id))

    def set_remote_server_param(self, id, param, value):
        self.set_item("{}/remote-servers[id='{}']/{}".format(self.namespace, id, param), value)

    def get_remote_server_param(self, id, param):
        return self.get_item("{}/remote-servers[id='{}']/{}".format(self.namespace, id, param))

    def unset_remote_server_param(self, id, param):
        self.delete_item("{}/remote-servers[id='{}']/{}".format(self.namespace, id, param))

    def get_remote_servers(self):
        config = self.get_running_config(full_path=False)
        config_indexed = {}
        if 'remote-servers' not in config:
            return config_indexed
        for server in config['remote-servers']:
            config_indexed[server['id']] = server
        return config_indexed
